@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2014 Canonical, Ltd.
+ * Copyright (C) 2013-2015 Canonical, Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License version 3, as published by
@@ -20,13 +20,14 @@
 #include <QString>
 
 #include <mir_toolkit/common.h>
+#include <mir_toolkit/event.h>
 
 class QTouchEvent;
 
 const char *touchPointStateToString(Qt::TouchPointState state);
 QString touchEventToString(const QTouchEvent *ev);
 
-QString mirSurfaceAttribAndValueToString(MirSurfaceAttrib attrib, int value);
+QString mirSurfaceAttribAndValueToString(MirWindowAttrib attrib, int value);
 const char *mirSurfaceTypeToStr(int value);
 const char *mirSurfaceStateToStr(int value);
 const char *mirSurfaceFocusStateToStr(int value);
@@ -35,4 +36,13 @@ const char *mirMotionActionToStr(int value);
 
 const char *applicationStateToStr(int state);
 
+QString mirPointerEventToString(MirPointerEvent const* event);
+QString mirTouchEventToString(MirTouchEvent const* event);
+QString mirKeyboardEventToString(MirKeyboardEvent const* event);
+const char *mirTouchActionToString(MirTouchAction touchAction);
+
+const char *qtCursorShapeToStr(Qt::CursorShape shape);
+
+const char *unityapiMirStateToStr(int state);
+
 #endif // UBUNTUGESTURES_DEBUG_HELPER_H
