@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2015 Canonical, Ltd.
+ * Copyright (C) 2013-2017 Canonical, Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License version 3, as published by
@@ -16,9 +16,11 @@
 
 #include "mirserverintegration.h"
 
-#include <QtPlatformSupport/private/qgenericunixfontdatabase_p.h>
-#include <QtPlatformSupport/private/qgenericunixeventdispatcher_p.h>
-#include <QtPlatformSupport/private/qgenericunixservices_p.h>
+#include <QtGlobal>
+
+#include <QtFontDatabaseSupport/private/qgenericunixfontdatabase_p.h>
+#include <QtEventDispatcherSupport/private/qgenericunixeventdispatcher_p.h>
+#include <QtServiceSupport/private/qgenericunixservices_p.h>
 
 #include <qpa/qplatformwindow.h>
 #include <qpa/qplatformaccessibility.h>
@@ -26,8 +28,8 @@
 #include <qpa/qplatforminputcontextfactory_p.h>
 #include <qpa/qwindowsysteminterface.h>
 
-#include <QCoreApplication>
-#include <QOpenGLContext>
+#include <QGuiApplication>
+#include <QStringList>
 #include <QDebug>
 
 // Mir
@@ -36,13 +38,16 @@
 
 // local
 #include "clipboard.h"
-#include "display.h"
-#include "displaywindow.h"
 #include "miropenglcontext.h"
 #include "nativeinterface.h"
+#include "offscreensurface.h"
 #include "qmirserver.h"
+#include "platformscreen.h"
+#include "screensmodel.h"
+#include "screenplatformwindow.h"
 #include "services.h"
 #include "ubuntutheme.h"
+#include "logging.h"
 
 namespace mg = mir::graphics;
 using qtmir::Clipboard;
@@ -51,10 +56,8 @@ MirServerIntegration::MirServerIntegration()
     : m_accessibility(new QPlatformAccessibility())
     , m_fontDb(new QGenericUnixFontDatabase())
     , m_services(new Services)
-    , m_mirServer(new QMirServer(QCoreApplication::arguments()))
-    , m_display(nullptr)
+    , m_mirServer(QMirServer::create())
     , m_nativeInterface(nullptr)
-    , m_clipboard(new Clipboard)
 {
     // For access to sensors, qtmir uses qtubuntu-sensors. qtubuntu-sensors reads the
     // UBUNTU_PLATFORM_API_BACKEND variable to decide if to load a valid sensor backend or not.
@@ -72,12 +75,14 @@ MirServerIntegration::MirServerIntegration()
                      QCoreApplication::instance(), &QCoreApplication::quit);
 
     m_inputContext = QPlatformInputContextFactory::create();
+
+    // Default Qt behaviour doesn't match a shell's intentions, so customize:
+    qGuiApp->setQuitOnLastWindowClosed(false);
 }
 
 MirServerIntegration::~MirServerIntegration()
 {
     delete m_nativeInterface;
-    delete m_display;
 }
 
 bool MirServerIntegration::hasCapability(QPlatformIntegration::Capability cap) const
@@ -87,7 +92,7 @@ bool MirServerIntegration::hasCapability(QPlatformIntegration::Capability cap) c
     case OpenGL: return true;
     case ThreadedOpenGL: return true;
     case BufferQueueingOpenGL: return true;
-    case MultipleWindows: return false; // multi-monitor support
+    case MultipleWindows: return true; // multi-monitor support
     case WindowManagement: return false; // platform has no WM, as this implements the WM!
     case NonFullScreenWindows: return false;
     default: return QPlatformIntegration::hasCapability(cap);
@@ -98,45 +103,23 @@ QPlatformWindow *MirServerIntegration::createPlatformWindow(QWindow *window) con
 {
     QWindowSystemInterface::flushWindowSystemEvents();
 
-    DisplayWindow* displayWindow = nullptr;
-
-    auto const mirServer = m_mirServer->mirServer().lock();
-    mg::DisplayBuffer* first_buffer{nullptr};
-    mg::DisplaySyncGroup* first_group{nullptr};
-    if (mirServer) {
-        mirServer->the_display()->for_each_display_sync_group([&](mg::DisplaySyncGroup &group) {
-            if (!first_group) {
-                first_group = &group;
-            }
-            group.for_each_display_buffer([&](mg::DisplayBuffer &buffer) {
-                if (!first_buffer) {
-                    first_buffer = &buffer;
-                }
-            });
-        });
+    auto screens = m_mirServer->screensModel();
+    if (!screens) {
+        qCritical("Screens are not initialized, unable to create a new QWindow/ScreenPlatformWindow");
+        return nullptr;
     }
 
-    // FIXME(gerry) this will go very bad for >1 display buffer
-    if (first_group && first_buffer)
-        displayWindow = new DisplayWindow(window, first_group, first_buffer);
-
-    if (!displayWindow)
-        return nullptr;
-
-    //displayWindow->requestActivateWindow();
-    return displayWindow;
+    return new ScreenPlatformWindow(window, screens->compositing());
 }
 
-QPlatformBackingStore *MirServerIntegration::createPlatformBackingStore(QWindow *window) const
+QPlatformBackingStore *MirServerIntegration::createPlatformBackingStore(QWindow */*window*/) const
 {
-    qDebug() << "createPlatformBackingStore" << window;
     return nullptr;
 }
 
 QPlatformOpenGLContext *MirServerIntegration::createPlatformOpenGLContext(QOpenGLContext *context) const
 {
-    qDebug() << "createPlatformOpenGLContext" << context;
-    return new MirOpenGLContext(m_mirServer->mirServer(), context->format());
+    return m_mirServer->createPlatformOpenGLContext(context);
 }
 
 QAbstractEventDispatcher *MirServerIntegration::createEventDispatcher() const
@@ -144,20 +127,50 @@ QAbstractEventDispatcher *MirServerIntegration::createEventDispatcher() const
     return createUnixEventDispatcher();
 }
 
+void MirServerIntegration::handleScreenAdded(QPlatformScreen *screen)
+{
+    // TODO: remove this after we no longer support Qt < 5.13
+    #if QT_VERSION < QT_VERSION_CHECK(5, 13, 0)
+            this->screenAdded(screen);
+    #else
+            QWindowSystemInterface::handleScreenAdded(screen);
+    #endif
+}
+
+void MirServerIntegration::handleScreenRemoved(QPlatformScreen *screen)
+{
+    // TODO: remove this after we no longer support Qt < 5.13
+    #if QT_VERSION < QT_VERSION_CHECK(5, 13, 0)
+            this->destroyScreen(screen);
+    #else
+            QWindowSystemInterface::handleScreenRemoved(screen);
+    #endif
+}
+
 void MirServerIntegration::initialize()
 {
     // Creates instance of and start the Mir server in a separate thread
-    if (!m_mirServer->start()) {
-        exit(2);
+    m_nativeInterface = new NativeInterface(m_mirServer.data());
+    m_mirServer->start();
+
+    auto screens = m_mirServer->screensModel();
+    if (!screens) {
+        qFatal("ScreensModel not initialized");
     }
 
-    m_display = new Display(m_mirServer->mirServer().data()->the_display()->configuration());
-    m_nativeInterface = new NativeInterface(m_mirServer->mirServer());
+    // need to create the screens before the integration initialises.
+    screens->update();
 
-    for (QPlatformScreen *screen : m_display->screens())
-        screenAdded(screen);
+    QObject::connect(screens.get(), &ScreensModel::screenAdded,
+            [this](PlatformScreen *screen) { handleScreenAdded(screen); });
+    QObject::connect(screens.get(), &ScreensModel::screenRemoved,
+            [this](PlatformScreen *screen) {
+        handleScreenRemoved(screen);
+    });
 
-    m_clipboard->setupDBusService();
+    Q_FOREACH(auto screen, screens->screens()) {
+        handleScreenAdded(screen);
+    }
 }
 
 QPlatformAccessibility *MirServerIntegration::accessibility() const
@@ -172,13 +185,15 @@ QPlatformFontDatabase *MirServerIntegration::fontDatabase() const
 
 QStringList MirServerIntegration::themeNames() const
 {
-    return QStringList(UbuntuTheme::name);
+    return QStringList(QLatin1String(UbuntuTheme::name));
 }
 
 QPlatformTheme *MirServerIntegration::createPlatformTheme(const QString& name) const
 {
-    Q_UNUSED(name);
-    return new UbuntuTheme;
+    if (name == QLatin1String(UbuntuTheme::name))
+        return new UbuntuTheme;
+
+    return QGenericUnixTheme::createUnixTheme(name);
 }
 
 QPlatformServices *MirServerIntegration::services() const
@@ -191,7 +206,24 @@ QPlatformNativeInterface *MirServerIntegration::nativeInterface() const
     return m_nativeInterface;
 }
 
+#ifdef WITH_CONTENTHUB
 QPlatformClipboard *MirServerIntegration::clipboard() const
 {
-    return m_clipboard.data();
+    static QPlatformClipboard *clipboard = nullptr;
+    if (!clipboard) {
+        clipboard = new Clipboard;
+    }
+    return clipboard;
+}
+#endif
+
+QPlatformOffscreenSurface *MirServerIntegration::createPlatformOffscreenSurface(
+        QOffscreenSurface *surface) const
+{
+    return new OffscreenSurface(surface);
+}
+
+Qt::WindowState MirServerIntegration::defaultWindowState(Qt::WindowFlags) const
+{
+    return Qt::WindowFullScreen;
 }
